/*
 * @(#)GallowsMenu.java    2.00 2000/12/30
 *
 */
import java.awt.*;
import java.awt.event.*;
import java.util.*;


/**
 * Menu pro applet hry sibenice
 *
 * @version 2.00 2000/12/30
 * @author Martin Kot
 */
public class GallowsMenu extends MenuBar{
    //Konstanty pro texty menu
    /** Vybira z vektoru textu menu polozky pro menu "Game" */
    protected static final int GAME = 0;
    /** Vybira z vektoru textu menu polozky pro menu "Help" */
    protected static final int HELP = 1;
    /** Vybira z vektoru textu menu text pro polozku "New game" */
    protected static final int NEW = 1;
    /** Vybira z vektoru textu menu text pro polozku "Solution" */
    protected static final int SOLUT = 2;
    /** Vybira z vektoru textu menu text pro polozku "Quit" */
    protected static final int QUIT = 3;
    /** Vybira z vektoru textu menu text pro polozku "Game help" */
    protected static final int GHELP = 1;
    /** Vybira z vektoru textu menu text pro polozku "About" */
    protected static final int ABOUT = 2;
    /** Texty pro menu */
    protected final String[][] menuTitles =
        {{"Game", "New game", "Show solution", "Quit"},
         {"Help", "Game help", "About"}};
    /** Nazev polozky s jazyky */
    protected final String LANG_TITLE = "Languages";
    /** Applet, ke ktery reaguje na prikazy menu */ 
    protected GallowsApplet galApplet;
    /** Posluchac udalosti od menu */
    protected ActionListener actListener;

    /**
     * Konstruktor
     *
     * @param galApplet applet, ve kterem bude menu pouzito
     * @param langs trida hledajici nazvy jazyku v XML souboru 
     *
     */
    public GallowsMenu(GallowsApplet galApplet, Languages langs) {
        this(galApplet);
        makeLangMenu(langs);
    }
 
    /**
     * Konstruktor
     *
     * @param galApplet applet, ve kterem bude menu pouzito
     *
     */
    public GallowsMenu(GallowsApplet galApplet) {
        super();
        this.galApplet = galApplet;

        Menu menu[] = new Menu[menuTitles.length];
        int itemsCount = 0;
        int k = 0;

        //Spocita pocet polozek
        for (int i = 0; i < menuTitles.length; i++) {
            for (int j = 1; j < menuTitles[i].length; j++) {
                itemsCount++;
            }
        }

        //Tvorba menu
        MenuItem items[] = new MenuItem[itemsCount];

        for (int i = 0; i < menuTitles.length; i++) {
            //Tvori nadpis menu
            menu[i] = new Menu(menuTitles[i][0]);
            this.add(menu[i]);

            for (int j = 1; j < menuTitles[i].length; j++) {
                //Tvori polozky menu
                items[k]=new MenuItem(menuTitles[i][j]);
                menu[i].add(items[k]);
                k++;
            }
        }

        //Vytvoreni posluchace udalosti od menu
        actListener = new ActionListener() {
            public void actionPerformed (ActionEvent ae) {
                if (ae.getActionCommand() == menuTitles[GAME][NEW]) {
                    changeGuessedWord();
                }
                else if (ae.getActionCommand() == menuTitles[GAME][QUIT]) {
                    endGame();
                }
                else if (ae.getActionCommand() == menuTitles[GAME][SOLUT]) {
                    showSolution();
                }
                else if (ae.getActionCommand() == menuTitles[HELP][ABOUT]) {
                    HelpFrame help = new HelpFrame(HelpFrame.ABOUT);
                }
                else if (ae.getActionCommand() == menuTitles[HELP][GHELP]) {
                    HelpFrame help = new HelpFrame(HelpFrame.GAME_HELP);
                }
            }
        };


        //Prida posluchace pro obsluhovani akci z tohoto menu
        for (int i = 0; i < items.length; i++) {
            items[i].addActionListener(actListener);
        }
    }
    /**
     * Tvori menu  
     * @param langs trida hledajici nazvy jazyku v XML souboru
     */ 
    protected void makeLangMenu(Languages langs){
        Vector v;
        Menu langMenu = new Menu(LANG_TITLE);
        this.add(langMenu);
	v = langs.getLanguages();
        MenuItem items[] = new MenuItem[v.size()];
        for (int j = 0; j < v.size(); j++) {
            //Tvori polozky menu
            items[j]=new MenuItem((String) v.elementAt(j));
            langMenu.add(items[j]);
        }
 
       //Vytvoreni posluchace udalosti od menu
        ActionListener langActListener = new ActionListener() {
            public void actionPerformed (ActionEvent ae) {
                changeLanguage(ae.getActionCommand());
            }
        };

        //Prida posluchace pro obsluhovani akci z tohoto menu
        for (int i = 0; i < items.length; i++) {
            items[i].addActionListener(langActListener);
        }


       
    }      

    /**
     * Reakce na volbu New Game
     *
     */
    protected void changeGuessedWord() {
        galApplet.changeGuessedWord();
    }


    /**
     * Reakce na volbu Quit
     *
     */
    protected void endGame() {
        galApplet.endGame();
    }


    /**
     * Reakce na volbu Show Solution
     *
     */
    protected void showSolution() {
        galApplet.showSolution();
    }

    /**
     * Reakce na volbu jazyka
     *
     */
    protected void changeLanguage(String actionCommand) {
        galApplet.changeLanguage(actionCommand);
    }
}


