/*
 * @(#)Languages.java       1.0 2001/01/01
 *
 */

import java.io.UnsupportedEncodingException;
import java.util.*;
import java.net.*;

import dom.wrappers.*;
import dom.DOMParserWrapper;
import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * Trida ziskavajici z XML souboru seznam jazyku a slov v nekterem z jazyku
 *
 * @version 1.00 2001/01/01
 * @author Martin Kot
 */
public class Languages {
   public static String VOCABULARY_TAG = "vocabulary";
   public static String WORDS_TAG = "words";
   public static String WORD_TAG = "word";
   public static String LANG_ATR = "language";
   
   /** Vector stringu - nazvu jazyku */
   protected Vector langs = new Vector();
   /** jmeno souboru XML */
   protected String uri = "vocabulary.xml";

   /** Vector boolean - jestli slova v originale obsahuji jen ASCII znaky */
   protected Vector ascv;
   /**
    * konstruktor 
    *
    * @exception Exception pri chybe parseru prochazejiciho XML soubor
    */
   public Languages() throws Exception{ 
     langs = fileToLangs();
   }

   /**
    * konstruktor 
    * @param url url asresa na ktere se nachazi soubor XML 
    *
    * @exception Exception pri chybe parseru prochazejiciho XML soubor
    */
   public Languages(URL url) throws Exception{ 
     uri = url.toString() + uri;
     langs = fileToLangs();
   }
   
   /**
    * konstruktor 
    * @param url url asresa na ktere se nachazi soubor XML 
    * @param fileName nazev XML souboru 
    *
    * @exception Exception pri chybe parseru prochazejiciho XML soubor
    */
   public Languages(URL url,String fileName) throws Exception{ 
     uri = url.toString() + fileName;
     langs = fileToLangs();
   }
   	
   /**
    * pomocna funkce na prevod NameNodeMap na pole Attr
    */
   protected Attr[] sortAttributes(NamedNodeMap attrs) {

      int len = (attrs != null) ? attrs.getLength() : 0;
      Attr array[] = new Attr[len];
      for ( int i = 0; i < len; i++ ) {
         array[i] = (Attr)attrs.item(i);
      }
      for ( int i = 0; i < len - 1; i++ ) {
         String name  = array[i].getNodeName();
         int    index = i;
         for ( int j = i + 1; j < len; j++ ) {
            String curName = array[j].getNodeName();
            if ( curName.compareTo(name) < 0 ) {
               name  = curName;
               index = j;
            }
         }
         if ( index != i ) {
            Attr temp    = array[i];
            array[i]     = array[index];
            array[index] = temp;
         }
      }


      return (array);

   } // sortAttributes(NamedNodeMap):Attr[]

   /**
    * Ziskava hodnotu uzlu
    *
    * @param node uzel, jehoz hodnotu je potreba ziskat
    * @return vector hodnot uzlu
    */
   protected Vector getNodeContext(Node node) {
      boolean canonical = true;
      Vector v = new Vector();
      // is there anything to do?
      if ( node == null ) {
         return v;
      }

      int type = node.getNodeType();
      switch ( type ) {
         // print document
         case Node.DOCUMENT_NODE: {
               v = getNodeContext(((Document)node).getDocumentElement());
               return v;
            }
            // print element with attributes
         case Node.ELEMENT_NODE: {
               NodeList children = node.getChildNodes();
               if ( children != null ) {
                  int len = children.getLength();
                  for ( int i = 0; i < len; i++ ) {
                     Vector v2 = getNodeContext(children.item(i)); 
                     for(int j=0;j<v2.size();j++) {
                       v.addElement(v2.elementAt(i));
		     }
                  }
               }
	       return v;
            }
            // print text
         case Node.TEXT_NODE: {
               v.addElement(normalize(node.getNodeValue()));
               return v;
            }
      }
      return v;
   } // print(Node)

    
    /** 
     * Ziskava vector jmen jazyku obsazenych v XML souboru. 
     * 
     * @return Vector retezcu - nazvu jazyku
     */
    protected Vector fileToLangs() throws Exception {
        Vector v = new Vector();
	Document document = null;
        DOMParserWrapper parser = (DOMParserWrapper) new DOMParser();
        document = parser.parse(uri);
        NodeList elements = document.getElementsByTagName(VOCABULARY_TAG);
        if (elements == null) {
            return v;
        }
        int elementCount = elements.getLength();
        for (int i = 0; i < elementCount; i++) {
             Element element = (Element)elements.item(i);
             NamedNodeMap attributes = element.getAttributes();
             if (attributes != null) {
                 int attributeCount = attributes.getLength();
                 for (int j = 0; j < attributeCount; j++) {
                    Attr attribute = (Attr)attributes.item(j);
                    if(attribute.getNodeName().compareTo(LANG_ATR) == 0){
                       v.addElement(normalize(attribute.getNodeValue()));
                    }
                 }
             }
        }
        return v;
    } 
    
    /**
     * Funkce vracejici nalezene nazvy jazyku 
     *
     * @return Vector retezcu - nazvu jazyku
     */
    public Vector getLanguages(){
       return langs;
    }


    /** 
     * Vraci vector informaci a slovech 
     * 
     * @return Vector booleanu - jestli slova v danem jazyce obsahuji 
     * jen ascii znaky
     */
     public Vector getAsciiWords() {
       return ascv;	
     }

    /** 
     * Vraci vector slov daneho jazyka 
     * 
     * @param langName nazev jazyka, ve kterem maji byt slova
     * @return Vector retezcu - slov daneho jazyka
     */
    public Vector getWords(String langName) throws Exception {
      Vector v = new Vector();
      ascv = new Vector();
      boolean asc = false;

      DOMParserWrapper parser = (DOMParserWrapper) new DOMParser();
      Document document = parser.parse(uri);

      NodeList elements = document.getElementsByTagName(VOCABULARY_TAG);
      if (elements == null) {
        return v;
      }
      int elementCount = elements.getLength();
      for (int i = 0; i < elementCount; i++) {
        Element element = (Element)elements.item(i);
        NamedNodeMap attributes = element.getAttributes();
        if (attributes != null) {
          int attributeCount = attributes.getLength();
          for (int j = 0; j < attributeCount; j++) {
            Attr attribute = (Attr)attributes.item(j);
            if(attribute.getNodeName().compareTo(LANG_ATR) == 0){
              if(normalize(attribute.getNodeValue()).compareTo(langName)==0){
                NodeList elementWords = element.getElementsByTagName(WORDS_TAG);
                int elemWordsCount = elementWords.getLength();
                for (int k = 0; k<elemWordsCount;k++){
                  Element el = (Element)elementWords.item(k);
                  NodeList elementWord = el.getElementsByTagName(WORD_TAG);
                  int elemWordCount = elementWord.getLength();
                  for (int l = 0; l<elemWordCount;l++){
                    Element e = (Element)elementWord.item(l);
                    NamedNodeMap attrs = e.getAttributes();
                    if (attrs != null) {
                      int attrCount = attrs.getLength();
                      for (int x = 0; x < attrCount; x++) {
                        Attr attr = (Attr)attrs.item(x);
                        if(attr.getNodeName().compareTo("ascii") == 0){
                          if(normalize(
			     attr.getNodeValue()).compareTo("yes")==0){
                             asc = true;
                          } else {
                             asc = false;
                          }
                          break;
                        }
                      }
                    }
                    else {
                      asc = false;
                    }
                    Vector v2 = getNodeContext(e);
                    for(int m = 0; m<v2.size();m++) {
                      String s = (String) v2.elementAt(m);
                      if (s.length() > GallowsGame.WORD_LENGTH) {
                        s = s.substring(0,GallowsGame.WORD_LENGTH);
		      }
		      for (int n = 0; n<s.length();n++) {
			if(!((s.charAt(n) >= 'a' && s.charAt(n) <='z') ||
                           (s.charAt(n) >= 'A' && s.charAt(n) <='Z'))) {
                           throw new BadCharactersException(
                              "Words in XML file contain unknown characters");
                        } 
                      }			
                      v.addElement(v2.elementAt(m));
                      ascv.addElement(new Boolean(asc));
                    }
                  }
                }
	      }
            }
          }
        }
      }
      return v;
    } 


    /**
     * Normalizuje dany string
     */
    private String normalize(String s) {
        StringBuffer str = new StringBuffer();

        int len = (s != null) ? s.length() : 0;
        for (int i = 0; i < len; i++) {
            char ch = s.charAt(i);
            switch (ch) {
                case '<': {
                    str.append("&lt;");
                    break;
                }
                case '>': {
                    str.append("&gt;");
                    break;
                }
                case '&': {
                    str.append("&amp;");
                    break;
                }
                case '"': {
                    str.append("&quot;");
                    break;
                }
                case '\r':
                case '\n': {
                    str.append("&#");
                    str.append(Integer.toString(ch));
                    str.append(';');
                    break;
                }
                default: {
                    str.append(ch);
                }
            }
        }

        return str.toString();

    } // normalize(String):String

} // class DOMFilter
